# About

This is as very simple docker configuration to run traefik locally for easy
project handling. Traefik allows you to run multiple projects simultaneously
without open port collisions.

Project tasks are handled using the TEAM23 b5 Task Runner (https://git.team23.de/build/b5).
See build/Taskfiles for the available tasks and their definition. The project follows our
common task schema (https://git.team23.de/build/b5/blob/master/docs/03_common_tasks.md). For
simple docker/traefik usage head over to https://git.team23.de/build/b5-docker-traefik.

## More background

Normally when using docker you might end up for example using http://localhost:8000/
for accessing the web service of your configuration. This might be an apache server or
just a development server you use.

When starting multiple projects all needing to bind to localhost:8000 this will - of
course - fail. So you tend to constantly stopping and starting projects for project
switching.

Traefik is meant to solve this by providing an HTTP proxy which automatically changes
its own configuration whenever you spin up new docker containers. It then allows access
to the web frontends by just passing all requests through itself, only needing one
public port for all projects. To differentiate the projects it uses local domain names.

So for example:
* http://web.project1.t23dev/ could use the web server of project1
* http://web.someotherproject.t23dev/ could use the web server of someotherproject

# Install

Installing the traefik proxy is pretty simple, as it follows the normal b5 project
schema. So to get things up and running you just **need to have docker installed**
and then:

## Initial setup

```bash
b5 install
b5 update
```

## Running the project

```bash
b5 run
# Recommendation, use "-d" to put traefik into the background and keep it running
# b5 run -d
```

## Housekeeping

```bash
b5 update
```

# Your projects with traefik

To use traefik you will need to setup a local domain and then change the project
configuration a little bit. I will try to get you through all the necessary steps:

## Providing a local domain (.t23dev)

Traefik needs a local domain to differentiate the projects you use. The domain name
will make traefik choose the right backend container.

We use dnsmasq as a local nameserver for accomplishing that. Setup is as follows:
```shell
$ brew update  # make sure we are up to date
$ brew install dnsmasq  # install dnsmasq
$ cd $(brew --prefix)  # cd into /usr/local
$ mkdir -p etc  # make sure /usr/local/etc exists
$ echo 'address=/.t23dev/127.0.0.1' > etc/dnsmasq.conf  # setup local dns resolving
$ sudo brew services start dnsmasq  # Enable dnsmasq daemon
$ sudo mkdir -p /etc/resolver  # make sure /etc/resolver exists
$ sudo bash -c 'echo "nameserver 127.0.0.1" > /etc/resolver/t23dev'  # tell macOS to use the local dnsmasq nameserver for .t23dev
```

**Optional:** If traefik does not work add this line to "/etc/hosts". Also do this when docker commands respond very slow.
```shell
127.0.0.1   localunixsocket.local localunixsocket
```

**Important:** A reboot might be needed for changes to take effect.

**Note:** The domain .t23dev shall be used for local development, see "T³P 0010 - Lokale Development Domains":
https://docs.google.com/document/d/1Pej_gE4yZVg0-vDY1azStgwHvq9J_oj-a4OrUbLpeSY/edit

## Example project configuration

To get your projects running with traefik I suggest the following schema:
* docker-compose.yml: Use this file to define all the services you need. DO NOT
  open any ports here.
* docker-compose.localhost.yml: This file for opening the local ports, like
  localhost:8000. This file can be used by anyone not caring about traefik and
  should be the default configuration.
* docker-compose.traefik.yml: Neccessary changes to the configuration to get traefik
  up and running. Basically adding the neccessary network and label options.

With b5 you should use `docker_compose_config_overrides` to choose the used additional
configuration file. This should default to 'localhost', developers might use local.yml
to change it to 'traefik'.

### config.yml

```yaml
# This is the central configuration for all tools we use/execute. It is parsed inside
# the Taskfile, too. See $CONFIG_project_name for example.
project:
  name: Example traefik project
  key: traefik_example
  url: http://www.doesnotexistyet.com/
paths:
  web: ../web
modules:
  # THIS is the important bit. Loading docker-compose.localhost.yml
  docker:
    docker_compose_config_overrides:
      - localhost
```

### docker-compose.yml

**Note:** I use the official phpmyadmin container for this example, as it is ready to
use without any complex configuration (only one service, no volumes, …). For example
purposes I skipped the neccessary MySQL server.

```yaml
version: '3'

services:
  phpmyadmin:
    image: phpmyadmin/phpmyadmin
```

### docker-compose.localhost.yml

In this file we open the normal ports for localhost:PORT usage. As a convention
we use port 8001 for phpmyadmin.

```yaml
version: '3'

services:
  phpmyadmin:
    ports:
      - "8001:80"
```

### docker-compose.traefik.yml

Now we add the configuration necessary for traefik to pick up the service and
provide the proxy services we want.

See https://docs.traefik.io/configuration/backends/docker/#labels-overriding-default-behaviour
for a full documentation of the possible labels you might use.

```yaml
version: '3'

services:
  phpmyadmin:
    # Make sure the service is inside the default network to access for example
    # mysql and the traefik gateway network to allow traefik accessing it.
    networks:
      - default
      - traefik_gateway
    # These labels are read by traefik for service configuration
    labels:
      # Enable traefik proxy
      - traefik.enable=true
      # Ensure traefik uses the right network
      - traefik.docker.network=traefik_gateway
      # It might be neccessary to point traefik to the correct port of the
      # service
      #- traefik.http.services.${COMPOSE_PROJECT_NAME}_phpmyadmin.loadbalancer.server.port=8000
  solr:
networks:
  # This is the normal default network
  default:
  # In addition we need to add the traefik network as external
  # (This also means traefik has to be started BEFORE your project)
  traefik_gateway:
    external: true
```

### Accessing phpmyadmin

If you setup a new project using this configuration it woll normally provide
http://localhost:8001/ for accessing phpmyadmin. This is ok, as some colleges might not
need/want to use traefik. For using traefik add an local.yml with the following contents:

```yaml
modules:
  docker:
    docker_compose_config_overrides:
      - traefik
```

After this (re)start the container. Then visit http://localhost:8080/ for accessing the
traefik status page. The phpmyadmin container should not be listed on the left side and
show you the Host 'phpmyadmin.traefikexample.t23dev' as its domain name.

Now just use http://phpmyadmin.traefikexample.t23dev/ to access it.

**Note:** when typing in the domain name yourself Google Chrome might open up
the normal google search as .t23dev is no common domain name. Type a trailing slash
to force the browser to use HTTP directly. So open up 'phpmyadmin.traefikexample.t23dev/'
in your addess bar will work.

# Trust the traefik certificate

* Open up Keychain Access. You can get to it from Application/Utilities/Keychain Access.app.
* Drag the certificate (see build/cert/*.crt) into Keychain Access.
* Go into the Certificates section and locate the certificate you just added
* Double click on it, enter the trust section and under “When using this certificate” select “Always Trust”

